Package: multiModTest
Type: Package
Title: Information Assessment for Individual Modalities in Multimodal
        Regression Models
Version: 1.0
Authors@R: c(
    person("Wanting", "Jin", email = "jinwanting5@gmail.com", role = c("aut", "cre")),
    person("Quefeng", "Li", email = "quefeng@email.unc.edu", role = "aut")
    )
Description: Provides methods for quantifying the information gain contributed by individual 
	modalities in multimodal regression models. Information gain is measured using Expected 
	Relative Entropy (ERE) or pseudo-R² metrics, with corresponding p-values and confidence 
	intervals. Currently supports linear and logistic regression models with plans for 
	extension to additional Generalized Linear Models and Cox proportional hazard model.
License: GPL-3
Encoding: UTF-8
RoxygenNote: 7.3.2
Imports: tidyverse, MASS, SIS, glmnet, ncvreg, MBESS, survival, dplyr
Depends: R (>= 3.6.0)
NeedsCompilation: no
Packaged: 2025-08-29 20:33:59 UTC; 10518
Author: Wanting Jin [aut, cre],
  Quefeng Li [aut]
Maintainer: Wanting Jin <jinwanting5@gmail.com>
LazyData: true
Repository: CRAN
Date/Publication: 2025-09-03 21:30:02 UTC
Built: R 4.6.0; ; 2026-04-30 20:52:52 UTC; unix

Package: multiModTest
Type: Package
Title: Information Assessment for Individual Modalities in Multimodal
        Regression Models
Version: 1.1
Authors@R: c(
    person("Wanting", "Jin", email = "jinwanting5@gmail.com", role = c("aut", "cre")),
    person("Quefeng", "Li", email = "quefeng@email.unc.edu", role = "aut")
    )
Description: Provides methods for quantifying the information gain contributed by individual 
	modalities in multimodal regression models. Information gain is measured using Expected 
	Relative Entropy (ERE) or pseudo-R² metrics, with corresponding confidence 
	intervals. Currently supports linear regression, logistic regression, and the Cox proportional
	hazards model. A robust Median-of-Means based estimator is also provided
	for heavy-tailed responses under the Gaussian and Negative-Binomial families, with basic
	bootstrap confidence intervals.
License: GPL-3
Encoding: UTF-8
RoxygenNote: 7.3.2
Imports: MASS, SIS, glmnet, ncvreg, MBESS, survival, dplyr
Suggests: testthat (>= 3.0.0)
Config/testthat/edition: 3
Depends: R (>= 3.6.0)
NeedsCompilation: no
Packaged: 2026-08-01 01:11:49 UTC; 10518
Author: Wanting Jin [aut, cre],
  Quefeng Li [aut]
Maintainer: Wanting Jin <jinwanting5@gmail.com>
LazyData: true
Repository: CRAN
Date/Publication: 2026-08-01 02:50:02 UTC
Built: R 4.5.2; ; 2026-08-01 03:45:13 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(print,summary.mglm.test)
S3method(summary,mglm.test)
export(mglm.test)
import(stats)
importFrom(MASS,glm.nb)
importFrom(MASS,mvrnorm)
importFrom(MBESS,conf.limits.nc.chisq)
importFrom(SIS,SIS)
importFrom(dplyr,case_when)
importFrom(glmnet,cv.glmnet)
importFrom(glmnet,glmnet)
importFrom(ncvreg,cv.ncvreg)
importFrom(ncvreg,cv.ncvsurv)
importFrom(ncvreg,ncvreg)
importFrom(ncvreg,ncvsurv)
importFrom(stats,BIC)
importFrom(stats,sd)
importFrom(survival,Surv)
importFrom(survival,coxph)
importFrom(utils,capture.output)

#' Example Dataset
#'
#' A toy dataset to demonstrate running this package on multimodal linear models.
#'
#' @format A data object that contains
#' \describe{
#'   \item{\code{y}}{A vector of 200 observations of continuous outcomes.}
#'   \item{\code{X}}{A 200 \eqn{\times} 600 matrix containing all training data.}
#'   \item{\code{mod.idx}}{A list of modality indices.}
#' }
"data_linear_model"


#' Example Dataset
#'
#' A toy dataset to demonstrate running this package on multimodal logistic models.
#'
#' @format A data object that contains
#' \describe{
#'   \item{\code{y}}{A vector 200 observations of outcomes. (0 or 1)}
#'   \item{\code{X}}{A 200 \eqn{\times} 600 matrix containing all training data.}
#'   \item{\code{mod.idx}}{A list of modality indices.}
#' }
"data_logistic_model"


#' Example Dataset
#'
#' A toy dataset to demonstrate running this package on multimodal Cox
#' proportional hazards models. Survival times follow a Cox model with
#' independent right censoring (about 27\% of the observations are censored).
#'
#' @format A data object that contains
#' \describe{
#'   \item{\code{y}}{A 200 \eqn{\times} 2 matrix of survival outcomes, with columns
#'   \code{time} (the observed follow-up time) and \code{status} (the event
#'   indicator: 1 = event, 0 = censored). It may be passed directly to
#'   \code{\link{mglm.test}}, or converted with \code{\link[survival]{Surv}}.}
#'   \item{\code{X}}{A 200 \eqn{\times} 800 matrix containing all training data.}
#'   \item{\code{mod.idx}}{A list of modality indices: columns 1--400 form
#'   modality 1 and columns 401--800 form modality 2.}
#' }
"data_cox_model"


#' Example Dataset
#'
#' A toy dataset to demonstrate robust (Median-of-Means) estimation on
#' multimodal Negative-Binomial models with heavy-tailed responses. The counts
#' are strongly overdispersed (variance-to-mean ratio near 19, with roughly
#' half of the observations equal to zero).
#'
#' @format A data object that contains
#' \describe{
#'   \item{\code{y}}{A vector of 300 observations of non-negative counts.}
#'   \item{\code{X}}{A 300 \eqn{\times} 600 matrix containing all training data.}
#'   \item{\code{mod.idx}}{A list of modality indices: columns 1--200, 201--400,
#'   and 401--600 form modalities 1, 2, and 3.}
#'   \item{\code{theta}}{The true Negative-Binomial dispersion used to generate
#'   the counts, \eqn{\theta = 0.5}. It may be supplied to \code{\link{mglm.test}}
#'   through the \code{theta} argument; otherwise it is estimated from the data.}
#' }
"data_nb_model"

## ===========================================================================
## Median-of-Means gradient-descent estimation of the modality
## information gain H_m, for heavy-tailed responses.
## ===========================================================================

## ---- Per-observation log-likelihood at beta (family-dispatched) ------------
##
## Returns (1/n) * logL(beta), up to an additive constant that does not depend
## on beta and therefore cancels in the H_m difference ll_full - ll_reduced.
## An intercept column is prepended internally, so beta = c(intercept, coefs)
## has length ncol(X) + 1.
##   gaussian          : profile (concentrated-variance) log-likelihood,
##                       (1/n) logL = -1/2 * log(RSS / n)  (+ const)
##   negative.binomial : NB log-likelihood at fixed dispersion theta.
## A design matrix with zero columns encodes the intercept-only (null) model.
## (median, quantile, var and sd come from 'stats', imported package-wide.)
mom_loglik <- function(X, y, beta, family, theta = NULL) {
  Xa  <- cbind(1, as.matrix(X))                     # prepend intercept column
  eta <- as.vector(Xa %*% beta)                     # beta = c(intercept, coefs)
  if (family == "gaussian") {
    rss <- max(sum((y - eta)^2), 1e-12)
    return(-0.5 * log(rss / length(y)))
  }
  if (family == "negative.binomial") {
    eta <- pmax(pmin(eta, 10), -10)                 # clamp for mu overflow
    mu  <- exp(eta)
    ll  <- sum(lgamma(y + theta) - lgamma(theta) - lgamma(y + 1) +
                 theta * log(theta / (theta + mu)) +
                 y     * log(mu    / (theta + mu)))
    return(ll / length(y))
  }
  stop("mom_loglik: 'family' must be 'gaussian' or 'negative.binomial'.")
}

## ---- Score of (1/n_blk) logL w.r.t. beta on one block ---
##
## An intercept column is prepended internally; the returned gradient has length
## ncol(X) + 1 and matches the beta = c(intercept, coefs) parameterisation used
## by mom_loglik.
## Gaussian          : g = Xa' (y - Xa beta) / n_blk
## Negative-binomial : g = Xa' r / n_blk,  r_i = theta (y_i - mu_i)/(mu_i + theta)
mom_score <- function(X, y, beta, family, theta = NULL) {
  Xa    <- cbind(1, as.matrix(X))                   # prepend intercept column
  n_blk <- nrow(Xa)
  eta   <- as.vector(Xa %*% beta)
  if (family == "gaussian") {
    r <- y - eta
    return(as.vector(crossprod(Xa, r)) / n_blk)
  }
  if (family == "negative.binomial") {
    eta <- pmax(pmin(eta, 10), -10)
    mu  <- exp(eta)
    r   <- theta * (y - mu) / (mu + theta)
    return(as.vector(crossprod(Xa, r)) / n_blk)
  }
  stop("mom_score: 'family' must be 'gaussian' or 'negative.binomial'.")
}

## ---- MoM gradient descent: median-log-likelihood block selection -----------
mom_gd <- function(X, y, family, theta = NULL, nblock = 5,
                   lr = NULL, maxit = NULL, tol = 1e-5) {
  X <- as.matrix(X)
  q <- ncol(X)
  n <- nrow(X)

  ## family-specific defaults (match the reference simulation settings)
  if (is.null(lr))    lr    <- switch(family, gaussian = 0.02,
                                      negative.binomial = 1e-3, 0.01)
  if (is.null(maxit)) maxit <- switch(family, gaussian = 1000L,
                                      negative.binomial = 500L, 1000L)

  K      <- max(1L, min(as.integer(nblock), n))
  fold   <- sort(rep_len(seq_len(K), n))            # contiguous near-equal blocks
  blocks <- split(seq_len(n), fold)
  beta   <- rep(0, q + 1L)                          # cold start (intercept + coefs)

  for (it in seq_len(maxit)) {
    old <- beta
    ll  <- vapply(blocks, function(idx)
      mom_loglik(X[idx, , drop = FALSE], y[idx], beta, family, theta),
      numeric(1))
    m_idx <- which.min(abs(ll - median(ll)))        # median-log-lik block
    idx   <- blocks[[m_idx]]
    g     <- mom_score(X[idx, , drop = FALSE], y[idx], beta, family, theta)
    beta  <- beta + lr * g
    if (max(abs(beta - old)) < tol) break
  }
  beta
}

## ---- MoM-GD point estimate of H_m ------------------------------------------
##
## Fits the full (all selected features) and reduced (selected features in the
## conditioning modalities only) models by MoM and returns the per-
## observation log-likelihood gain ll_full - ll_reduced.
mom_Th <- function(X, y, var_id, Mbeta, family, theta = NULL, nblock = 5) {
  sel_full <- var_id
  sel_red  <- var_id[var_id %in% Mbeta]
  Xf <- X[, sel_full, drop = FALSE]
  Xr <- X[, sel_red,  drop = FALSE]
  bf <- mom_gd(Xf, y, family, theta, nblock)
  br <- mom_gd(Xr, y, family, theta, nblock)        # intercept-only if reduced is null
  mom_loglik(Xf, y, bf, family, theta) -
    mom_loglik(Xr, y, br, family, theta)
}

## ---- Basic (pivotal) bootstrap CI for the MoM estimate of H_m -----------
##
## Resamples observations with replacement 'nboot' times, recomputing the full
## MoM pipeline on each resample, and inverts the bootstrap distribution
## with the basic bootstrap:
##   two.sided : [ 2*Th - q_{1-a/2},  2*Th - q_{a/2} ]
##   one.sided : [ 2*Th - q_{1-a},    Inf )
## The lower bound is floored at 0 because H_m >= 0 (keeps pseudo-R^2 valid).
mom_boot_ci <- function(X, y, var_id, Mbeta, family, theta = NULL, nblock = 5,
                        Th, nboot = 200, conf.level = 0.95,
                        CI.type = c("two.sided", "one.sided")) {
  CI.type <- match.arg(CI.type)
  n     <- nrow(X)
  alpha <- 1 - conf.level
  Tb    <- rep(NA_real_, nboot)

  for (b in seq_len(nboot)) {
    id <- sample.int(n, n, replace = TRUE)
    Tb[b] <- tryCatch(
      mom_Th(X[id, , drop = FALSE], y[id], var_id, Mbeta, family, theta, nblock),
      error = function(e) NA_real_)
  }
  Tb <- Tb[is.finite(Tb)]
  if (length(Tb) < 2L) return(c(NA_real_, NA_real_))

  if (CI.type == "two.sided") {
    q  <- quantile(Tb, c(alpha / 2, 1 - alpha / 2), na.rm = TRUE)
    ci <- c(2 * Th - q[2], 2 * Th - q[1])
  } else {
    q  <- quantile(Tb, conf.level, na.rm = TRUE)
    ci <- c(2 * Th - q, Inf)
  }
  ci[1] <- max(ci[1], 0)                             # H_m >= 0
  names(ci) <- NULL
  ci
}

## ---- MoM estimation wrapper --------
##
## Returns the same list shape as mod_est: num.nonzeros, Th, CI.  The bootstrap
## CI is computed only when CI = TRUE (default FALSE for the robust path).
mom_est <- function(X, y, var_id, Mbeta, family, theta = NULL, nblock = 5,
                    CI = FALSE, nboot = 200, conf.level = 0.95,
                    CI.type = c("two.sided", "one.sided")) {
  CI.type <- match.arg(CI.type)
  if (length(var_id) == 0L)
    return(list(num.nonzeros = 0, Th = 0, CI = c(NA, NA)))
  num.nonzeros <- length(var_id) - length(var_id[var_id %in% Mbeta])
  if (num.nonzeros == 0)
    return(list(num.nonzeros = 0, Th = 0, CI = c(NA, NA)))

  Th <- mom_Th(X, y, var_id, Mbeta, family, theta, nblock)

  ci <- c(NA, NA)
  if (isTRUE(CI))
    ci <- mom_boot_ci(X, y, var_id, Mbeta, family, theta, nblock,
                      Th = Th, nboot = nboot, conf.level = conf.level,
                      CI.type = CI.type)

  list(num.nonzeros = num.nonzeros, Th = Th, CI = ci)
}

## ---- Fixed-dispersion estimate for the Negative-Binomial family ------------
##
## When the user does not supply theta, estimate it once (on the screened
## design) via MASS::glm.nb, falling back to a method-of-moments estimate.
#' @importFrom MASS glm.nb
est_theta_nb <- function(y, Xsub) {
  th <- tryCatch(
    as.numeric(suppressWarnings(glm.nb(y ~ as.matrix(Xsub)))$theta),
    error = function(e) NA_real_)
  if (is.na(th) || !is.finite(th) || th <= 0) {
    mu <- max(mean(y), 0.01)
    v  <- max(var(y), mu + 0.01)
    th <- min(max(mu^2 / (v - mu), 0.01), 100)      # MoM fallback
  }
  th
}

## ERE / H_m Estimation and confidence intervals
#' @import stats
#' @importFrom survival coxph
mod_est <- function(X, y, var_id, Mbeta, family,
                    conf.level = 0.95, CI.type = c("two.sided","one.sided"),
                    CI = TRUE){
  n = nrow(X)
  # xname = paste("x",var_id, sep = ".")
  if(length(var_id)==0){
    return(list(num.nonzeros=0,Th=0,CI=c(NA,NA)))
  }
  num.nonzeros = length(var_id) - length(var_id[var_id %in% Mbeta])
  if(num.nonzeros==0){
    return(list(num.nonzeros=0,Th=0,CI=c(NA,NA)))
  }

  ## fit the full model
  xf = X[,var_id]
  ## fit the reduced model
  if(length(var_id[var_id %in% Mbeta])!=0){
    xr = X[,var_id[var_id %in% Mbeta]]
    }


  if(family == 'gaussian'){
    fit_full = lm(y~xf)
    if(length(var_id[var_id %in% Mbeta])==0){fit_reduced = lm(y~1)}
    else{
      fit_reduced = lm(y~xr)
    }
    # estimation using log likelihood
    Th = as.numeric((logLik(fit_full)-logLik(fit_reduced))/n)
  }
  if(family == 'binomial'){
    fit_full = glm(y~xf, family = 'binomial')
    if(length(var_id[var_id %in% Mbeta])==0){fit_reduced = glm(y~1, family = 'binomial')}
    else{
      fit_reduced = glm(y~xr, family = 'binomial')
    }
    Th = -(fit_full$deviance - fit_reduced$deviance) / (2*n)
  }
  if(family == 'cox'){
    ## y is a 'Surv' object; Th is the per-observation gain in the Cox partial
    ## log-likelihood contributed by the modality-specific features.
    fit_full = coxph(y ~ as.matrix(xf))
    if(length(var_id[var_id %in% Mbeta])==0){
      ## reduced model has no covariates: use the null partial log-likelihood
      ll_reduced = fit_full$loglik[1]
    }else{
      fit_reduced = coxph(y ~ as.matrix(xr))
      ll_reduced = as.numeric(logLik(fit_reduced))
    }
    Th = (as.numeric(logLik(fit_full)) - ll_reduced) / n
  }

  # Calculating Confidence Interval
  if(!isTRUE(CI)){
    return(list(num.nonzeros=num.nonzeros,Th=Th,CI=c(NA,NA)))
  }
  if(n*Th < 0.0001){
    return(list(num.nonzeros=num.nonzeros,Th=Th,CI=c(NA,NA)))
  }

  if(family == 'cox'){
    ## Cox partial-likelihood ratio: noncentrality = n * Th, scale = n
    if(CI.type == "two.sided"){
      CI = unlist(conf.limits.nc.chisq(Chi.Square=2*n*Th, conf.level=conf.level,
                                       df=num.nonzeros)[c(1,3)])/(2*n)
    }
    else if(CI.type == "one.sided"){
      CI_L = unlist(conf.limits.nc.chisq(Chi.Square=2*n*Th, alpha.lower=1-conf.level,
                                         alpha.upper=0, conf.level=NULL,
                                         df=num.nonzeros)[2])/(2*n)
      CI = c(CI_L,Inf)
    }
  }else{
    ## Gaussian / Binomial deviance ratio: noncentrality = 2 * n * Th, scale = 2 * n
    if(CI.type == "two.sided"){
      CI = unlist(conf.limits.nc.chisq(Chi.Square=2*n*Th, conf.level=conf.level,
                                       df=num.nonzeros)[c(1,3)])/(2*n)
    }
    else if(CI.type == "one.sided"){
      CI_L = unlist(conf.limits.nc.chisq(Chi.Square=2*n*Th,
                                         alpha.lower=1-conf.level, alpha.upper=0,
                                         conf.level=NULL, df=num.nonzeros)[2])/(2*n)
      CI = c(CI_L,Inf)
    }
  }

  return(list(num.nonzeros=num.nonzeros,Th=Th,CI=CI))
}



## Iterative Dimension reduction procedure
# @import stats
# ISIS_mglm <- function(X, y, family, iter , tune = tune, iter.max = 20){
#   p = ncol(X)
#   n = nrow(X)
#
#   ix0 = c()
#   iterind = 0
#
#   ## Variable Selection through SIS
#   nseq = seq(12, ceiling(n/(log(n))),by = 1)
#   tune_stat = 10000
#   tune_value = rep(0,length(nseq))
#   var_id0 = c()
#
#   for (k in 1:length(nseq)){
#     model = SIS(X, y, family=family, tune=tune,nsis = nseq[k],iter = FALSE)
#     xf = X[,model$sis.ix0]
#     fit0 = glm(y~xf-1, family = family)
#     tune_value[k] = switch(tune, bic = BIC(fit0),
#                            ebic = BIC(fit0) + log(choose(p,k)),
#                            aic = AIC(fit0) )
#     if(tune_value[k] < tune_stat){
#       var_id0_min = model$sis.ix0
#       tune_stat = tune_value[k]
#     }
#   }
#   k = min(which(tune_value < min(tune_value) + sd(tune_value)/length(nseq) ))
#   model = SIS(X, y, family=family, tune=tune,nsis = nseq[k],iter = FALSE)
#   ix0 = model$sis.ix0
#
#   if(iter == FALSE){
#     ## SIS procedure
#     iterind = 1
#     cat("Iter", iterind, ", selection: ", c(ix0), "\n")
#   }else{
#     iterind = 1
#     cat("Iter", iterind, ", selection: ", c(ix0), "\n")
#     repeat{ ## Iterative screening procedure
#       iterind = iterind + 1
#       new.ind = 0
#       nsis = max(ceiling(n/(log(n)))-length(ix0),2)
#       nseq = seq(1, nsis,by = 1)
#       tune_value = rep(0,length(nseq))
#       candind = setdiff(1:p, ix0)
#
#       condcoef = margcoef(X, y, ix0, family = family, null.model = FALSE, iterind = iterind)
#       rankcoef = sort(condcoef, decreasing = TRUE, index.return = TRUE)
#       var_id_iter = ix0
#
#       for (k in 1:length(nseq)){
#         newix = candind[rankcoef$ix[1:nseq[k]]]
#         xf = X[,c(ix0,newix)]
#         fit0 = glm(y~xf-1, family = family)
#         tune_value[k] = switch(tune, bic = BIC(fit0),
#                                ebic = BIC(fit0) + log(choose(p,k)),
#                                aic = AIC(fit0) )
#         if(tune_value[k] < tune_stat){
#           new.ind = 1
#           var_id0_min = model$sis.ix0
#           tune_stat = tune_value[k]
#         }
#       }
#
#       if(new.ind==1){
#         k = min(which(tune_value < min(tune_value) + sd(tune_value)/length(nseq) ))
#         if(k==1) newix = c() else{
#           newix = candind[rankcoef$ix[1:nseq[k-1]]]
#         }
#         tune_stat = tune_value[k]
#       }else{
#         newix=c()
#       }
#       ix_iter = sort(c(ix0,newix))
#
#       cat("Iter", iterind, ", selection: ", ix_iter, "\n")
#       if (length(newix) >= nsis || iterind >= iter.max) {
#         ix0 = ix_iter
#         if (length(newix) >= nsis)
#           cat("Maximum number of variables selected \n")
#         if (iterind >= iter.max)
#           cat("Maximum number of iterations reached \n")
#         break
#       }
#       ix0 = ix_iter
#       if (iterind > 1 & length(newix) == 0) {
#         cat("Model already selected \n")
#         break
#       }
#     }
#   }
#   return(ix0)
# }

#' Summary method for objects of class "mglm.test"
#' @param object An \code{mglm.test} object.
#' @param ... Additional arguments that could be passed to \code{summary.mglm.test}.
#'
#' @return An object with S3 class \code{summary.mglm.test}. The class has its own print
#' method and contains the following list of elements.
#' \item{sum.ERE}{The summary table of point estimate and confidence interval
#' of ERE for each modality.}
#' \item{sum.R2}{The summary table of point estimate and confidence interval
#' of pseudo-\eqn{R^2} for each modality.}
#' \item{conf.level}{ Level of confidence intervals.}
#' \item{sel.mod}{ Index of the most informative modality.}
#' @export
summary.mglm.test <- function(object, ...) {
  args_extra <- list(...)
  nmod <- length(object$num.nonzeros)

  ## Checke if there is only one informative modality
  ixnum <- sapply(object$sel.idx,length)
  if(sum(ixnum==0) == nmod-1){
    k = order(ixnum,decreasing = TRUE)[1]
  }else{
    if(sum(ixnum)==0) {
      k = 0
    } else {
      k = order(object$ERE,decreasing=TRUE)[1]
    }
  }
  est_ERE <- data.frame(estimate = object$ERE,
                        lower.bound = object$ERE.CI.L,
                        upper.bound = object$ERE.CI.U,
                        row.names = paste("Modality",seq(1:nmod))
  )
  est_R2 <- data.frame(estimate = object$R2,
                       lower.bound = object$R2.CI.L,
                       upper.bound = object$R2.CI.U,
                       row.names = paste("Modality",seq(1:nmod))
  )

  summary_data <- list(sum.ERE = est_ERE, sum.R2 = est_R2, conf.level = object$conf.level,
                       sel.mod = k)
  class(summary_data) <- "summary.mglm.test"
  return(summary_data)

}


#' @param x  A \code{summary.mglm.test} object.
#'
#' @rdname summary.mglm.test
#' @export
print.summary.mglm.test <- function(x, ...) {
  if(x$sel.mod == 0){
    cat("No variable selected. \n\n")
  }else{
    cat("The most informative modality is modality", x$sel.mod, ".\n\n")
  }

  cat("Summary of mglm.test result:\n\n")
  cat("Expected Relative Entropy:\n\n")
  print(format(x$sum.ERE, digits = 4, justify = "right"))
  cat("The confidence level =", 100*x$conf.level,"% \n\n")

  cat("Pseudo R2:\n")
  print(format(x$sum.R2, digits = 4, justify = "right"))
  cat("The confidence level =", 100*x$conf.level,"% \n\n")
  invisible(x) # Return the object invisibly
}

loglik = function(X, y, beta, family) {
  K = dim(beta)[2]
  link = cbind(1, X) %*% beta
  yrep = repmat(y, 1, K)
  if (family == "gaussian")
    return(apply((yrep - link)^2, 2, sum))
  if (family == "poisson")
    return(apply(exp(link) - yrep * link, 2, sum))
  if (family == "binomial")
    return(apply(log(1 + exp(link)) - yrep * link, 2, sum))
}


repmat = function(X, m, n) {
  ## R equivalent of repmat (matlab)
  X = as.matrix(X)
  mx = dim(X)[1]
  nx = dim(X)[2]
  matrix(t(matrix(X, mx, nx * n)), mx * m, nx * n, byrow = T)
}


getdf = function(coef.beta) {
  apply(abs(coef.beta) > 1e-10, 2, sum)
}

margcoef <- function(x, y, condind = NULL, family, null.model = FALSE, iterind, method='SIS') {
  n = dim(x)[1]
  p = dim(x)[2]
  ones = rep(1, n)
  candind = setdiff(1:p, condind)
  if (iterind == 0) {
    if (family == "cox")
      margcoef = abs(cor(x, y[, 1])) else {
        margcoef = case_when(
          method=='SIS'~abs(cor(x, y)),
          # method=='RoSIS'~abs(mean(x*fy(y))),
          # method=='RRCS'~abs(cor(x, y,method = 'kendall')),
          TRUE~abs(cor(x, y))
        )
      }
  } else {
      if (null.model == TRUE) {
        if (is.null(condind) == TRUE) {
          x = x[sample(1:n), ]
        }
        if (is.null(condind) == FALSE) {
          x[, candind] = x[sample(1:n), candind]
        }
      }
      margcoef = abs(sapply(candind, mg, x, y, ones, family, condind,method))
    }


  return(margcoef)
}

mg <- function(index, x = x, y = y, ones = ones, family = family, condind = condind, method=method) {
  if(method=='SIS'){
    margfit = switch(family, gaussian = coef(glm.fit(cbind(ones, x[, index], x[, condind]), y, family = gaussian()))[2],
                     binomial = coef(glm.fit(cbind(ones, x[, index], x[, condind]), y, family = binomial()))[2],
                     poisson = coef(glm.fit(cbind(ones, x[, index], x[, condind]), y, family = poisson()))[2],
                     probit = coef(glm.fit(cbind(ones, x[, index], x[, condind]), y, family = binomial("probit")) )[2],
                     negbin = coef(glm.nb(y~cbind(ones, x[, index], x[, condind]), init.theta = stop))[2],
                     cox = coef(coxph(y ~ cbind(x[, index], x[, condind])))[1],
                     coxp = coef(coxph(y ~ cbind(x[, index], x[, condind])))[1]/
                            sqrt(vcov(coxph(y ~ cbind(x[, index], x[, condind])))[1,1])
    )
  }else{
    fy <- with(density(y), splinefun(x, cumsum(y) / sum(y), method="monoH.FC"))
    margfit = switch(method, RoSIS=mean(x[,c(index,condind)]*fy(y)),
                     RRCS = cor(x[,index], y,method = 'kendall'))
  }
}


## partially penalized function for SIS
#' @importFrom ncvreg cv.ncvsurv ncvsurv
tune.fit.adj <- function(x, y, family = c("gaussian", "binomial", "poisson", "cox"), penalty = c("SCAD", "MCP", "lasso"),
                         concavity.parameter = switch(penalty, SCAD = 3.7, 3),
                         tune = c("cv", "aic", "bic", "ebic"), nfolds = 10,
                         lambda = NULL, nlambda = 100,
                     type.measure = c("deviance", "class", "auc", "mse", "mae"), gamma.ebic = 0.5,
                     penalty.factor = rep(1, ncol(x))
                    ) {

  if (is.null(x) || is.null(y))
    stop("The data is missing!")

  x = as.matrix(x)

  this.call = match.call()
  family = match.arg(family)
  penalty = match.arg(penalty)
  # if (class(concavity.parameter) != "numeric")
  #   stop("concavity.parameter must be numeric!")
  # tune = match.arg(tune)
  # if (class(nfolds) != "numeric")
  #   stop("nfolds must be numeric!")
  type.measure = match.arg(type.measure)
  nobs = nrow(x)
  nvars = ncol(x)

  if (tune == "cv") {
    if (penalty == "lasso" ) {
      cv.fit = cv.glmnet(x, y, family = family, type.measure = type.measure, nfolds = nfolds)
      coef.beta = coef(cv.fit, s = "lambda.1se")
      reg.fit = cv.fit$glmnet.fit
      lambda = cv.fit$lambda.1se
      lambda.ind = which(cv.fit$lambda == cv.fit$lambda.1se)
    } else if (family != 'cox') {
      cv.fit = cv.ncvreg(x, y, family = family, penalty = penalty, gamma = concavity.parameter, nfolds = nfolds)
      cv.1se.ind = min(which(cv.fit$cve<cv.fit$cve[ cv.fit$min]+cv.fit$cvse[ cv.fit$min]))
      coef.beta = cv.fit$fit$beta[, cv.1se.ind]  # extract coefficients at a single value of lambda, including the intercept
      reg.fit = cv.fit$fit

      lambda = cv.fit$lambda[cv.1se.ind]
      lambda.ind = cv.1se.ind
    } else {
      cv.fit = cv.ncvsurv(x, y, family = family, penalty = penalty, gamma = concavity.parameter, nfolds = nfolds)
      cv.1se.ind = min(which(cv.fit$cve<cv.fit$cve[ cv.fit$min]+cv.fit$cvse[ cv.fit$min]))
      coef.beta = cv.fit$fit$beta[, cv.1se.ind]  # extract coefficients at a single value of lambda
      reg.fit = cv.fit$fit

      lambda = cv.fit$lambda[cv.1se.ind]
      lambda.ind = cv.1se.ind
    }
  } else {
    n = nrow(x)
    if (penalty == "lasso" ) {
      if(is.null(lambda)){
        reg.fit = glmnet(x, y, family = family, nlambda = nlambda,
                         penalty.factor = penalty.factor)
      }else{
        reg.fit = glmnet(x, y, family = family, lambda = lambda, nlambda = nlambda,
                         penalty.factor = penalty.factor)
      }
      coef.beta = rbind(reg.fit$a0,as.matrix(reg.fit$beta))  # extract coefficients at all values of lambda,  including the intercept
      dev = deviance(reg.fit)
      reg.df = reg.fit$df
    } else {
      if(family != 'cox'){
        if(is.null(lambda)){
        reg.fit = ncvreg(x, y, family = family, penalty = penalty, gamma = concavity.parameter,
                        nlambda = nlambda,
                         penalty.factor = penalty.factor)
        }else{
          reg.fit = ncvreg(x, y, family = family, penalty = penalty, gamma = concavity.parameter,
                           lambda=lambda, nlambda = nlambda,
                           penalty.factor = penalty.factor)
        }
        coef.beta = reg.fit$beta  # extract coefficients at all values of lambda, including the intercept
        dev = loglik(x, y, coef.beta, family = family)
        reg.df = getdf(coef.beta[-1, , drop = FALSE])
      } else {
        if(is.null(lambda)){
          reg.fit = ncvsurv(x, y, family = family, penalty = penalty,
                            gamma = concavity.parameter, nlambda = nlambda,
                            penalty.factor = penalty.factor)
        }else{
          reg.fit = ncvsurv(x, y, family = family, penalty = penalty,
                            gamma = concavity.parameter, lambda = lambda,
                            penalty.factor = penalty.factor)
        }
        coef.beta = reg.fit$beta  # extract coefficients at all values of lambda, including the intercept
        dev = 2*reg.fit$loss
        reg.df = getdf(coef.beta)
      }
    }

    if (tune == "aic") {
      obj = dev + 2 * reg.df
    }
    if (tune == "bic") {
      obj = dev + log(n) * reg.df
    }
    if (tune == "ebic") {
      obj = dev + log(n) * reg.df + 2 * gamma.ebic * log(choose(dim(x)[2], reg.df))
    }
    lambda.ind = which.min(obj)
    coef.beta = coef.beta[, lambda.ind]
    lambda = reg.fit$lambda[lambda.ind]
    obj = obj
  }

  if(family != 'cox'){
    a0 = coef.beta[1]
    coef.beta = coef.beta[-1]
  } else{
    a0 = NULL
    coef.beta = as.vector(coef.beta)
  }
  ix = which(coef.beta != 0)
  beta = coef.beta[ix]
  return(list(ix = ix, a0 = a0, beta = beta, fit = reg.fit, lambda = lambda, lambda.ind = lambda.ind, obj = obj, lambda.seq = reg.fit$lambda))
}



## ---------------------------------------------------------------------------
## Sure Independence Screening for the Cox proportional hazards model
## ---------------------------------------------------------------------------

## One screening step for the Cox model.
##
## Ranks the candidate features by the magnitude of their (conditional)
## marginal Cox utility and, for the iterative version, selects the subset that
## minimises the (extended) BIC of the resulting partial-likelihood fit.
##
## Arguments
##   x0          : the n x p feature matrix.
##   y0          : a 'Surv' object with the observed times and event indicators.
##   nsis        : maximum number of new features to consider in this step.
##   family      : marginal utility used for ranking, 'cox' (marginal
##                 coefficient) or 'coxp' (marginal Wald z-statistic).
##   candind     : indices of the candidate features (typically setdiff(1:p, ix0)).
##   BIC_old     : the current best (extended) BIC to improve upon.
##   ix0         : indices of the features already retained by previous steps.
##   iterind     : the current screening-iteration index (0 uses a correlation
##                 screen, >= 1 uses conditional marginal Cox fits).
##   sis_method  : 'isis' for the BIC-driven iterative selection, 'sis' for a
##                 single threshold-based pass on the marginal statistic.
##   gamman      : threshold on the marginal statistic used when sis_method='sis'.
#' @importFrom survival coxph Surv
#' @importFrom stats BIC sd
SIS_cox <- function(x0, y0, nsis, family, candind, BIC_old, ix0,
                    iterind, sis_method = 'isis', gamman = NA) {
  new.ind = 0
  p = ncol(x0)

  ## (conditional) marginal utility of every candidate feature
  mcoef = margcoef(x0, y0, ix0, family = family, null.model = FALSE,
                   iterind = iterind, method = 'SIS')
  rankcoef = sort(mcoef, decreasing = TRUE, index.return = TRUE)

  if (sis_method == 'isis') {

    nsis = min(nsis, length(candind))          ## never look past the candidates
    nseq = seq(1, nsis, by = 1)
    bic = rep(0, length(nseq) + 1)
    var_id0 = c()
    bic[1] = BIC_old

    for (sk in 1:length(nseq)) { ## grow the model and track the extended BIC
      newix = candind[rankcoef$ix[1:nseq[sk]]]
      xf = as.matrix(x0[, c(ix0, newix)])
      fit0 <- coxph(y0 ~ xf)
      eBIC = BIC(fit0) + log(choose(p, sk)) # extended BIC with gamma = 0.5
      bic[sk + 1] = eBIC
      if (BIC(fit0) < BIC_old) {
        new.ind = 1
        var_id0_min = newix
        BIC_old = BIC(fit0)
      }
    }

    if (new.ind == 1) {
      sk = min(which(bic < min(bic) + sd(bic) / sqrt(length(nseq))))
      if (sk == 1) var_id0 = c() else {
        var_id0 = sort(candind[rankcoef$ix[1:(sk - 1)]])
      }
      BIC_min = bic[sk]
    } else {
      var_id0 = c()
      BIC_min = BIC_old
    }

  } else { ## single threshold-based screening pass
    var_id0 = sort(candind[rankcoef$ix[rankcoef$x >= gamman]])
    if (length(var_id0) == 0) {
      BIC_min = BIC_old
    } else {
      xf = as.matrix(x0[, c(ix0, var_id0)])
      fit0 <- coxph(y0 ~ xf)
      BIC_min = BIC(fit0)
    }
  }

  return(list(BIC = BIC_min, ix0 = var_id0))
}


## (Iterative) Sure Independence Screening for the Cox model.
##
## Wraps 'SIS_cox' into the (iterative) screening loop used for dimension
## reduction before the partially penalised inference step.  Setting
## iter = FALSE performs a single BIC-driven SIS pass; iter = TRUE performs the
## full ISIS procedure.
##
## Arguments
##   X         : the n x p feature matrix.
##   y         : a 'Surv' object with the observed times and event indicators.
##   iter      : logical; whether to run the iterative (ISIS) procedure.
##   iter.max  : maximum number of screening iterations.
##   trace     : logical; whether to print the selection at each iteration.
##
## Value: a sorted integer vector of the retained feature indices.
#' @importFrom survival Surv
screen_cox <- function(X, y, iter = TRUE, iter.max = 20, trace = FALSE) {
  n = nrow(X)
  p = ncol(X)
  ix0 = c()
  iterind = 0
  BIC_old = 1e4

  repeat {
    iterind = iterind + 1
    if (length(setdiff(1:p, ix0)) == 0) break          ## no candidates left
    nsis = max(ceiling(n / log(n)) - length(ix0), 2)
    SIS_i = SIS_cox(x0 = X, y0 = y, nsis = nsis, family = 'cox',
                    candind = setdiff(1:p, ix0), BIC_old = BIC_old,
                    ix0 = ix0, iterind = iterind, sis_method = 'isis')
    ix1 = SIS_i$ix0
    BIC_old = SIS_i$BIC

    if (trace) cat("Iter", iterind, ", selection: ", c(ix0, ix1), "\n")

    ix0 = c(ix0, ix1)

    if (!iter) break                                   ## single SIS pass
    if (length(ix1) >= nsis || iterind >= iter.max) {  ## stop growing
      if (trace && length(ix1) >= nsis)
        cat("Maximum number of variables selected \n")
      if (trace && iterind >= iter.max)
        cat("Maximum number of iterations reached \n")
      break
    }
    if (iterind > 1 && length(ix1) == 0) {             ## model stabilised
      if (trace) cat("Model already selected \n")
      break
    }
  }

  return(sort(ix0))
}

#' @title Modality Assessment in Multimodal Generalized Linear Models
#'
#' @description Provides statistical inference for modality-specific information gain in
#' multimodal GLMs. Estimates ERE and pseudo-\eqn{R^2} with confidence intervals using
#' Sure Independence Screening for variable selection and penalized likelihood for inference.
#'
#'
#' @param X The finite numeric \eqn{n \times p} data matrix consisting of features from all modalities.
#' @param y The response. For \code{family = "gaussian"} or \code{"binomial"} this is the
#' length-\eqn{n} vector (or one-column matrix/data frame) of finite responses; binomial responses
#' must contain both 0 and 1.
#' Negative-binomial responses must be non-negative integer counts. For \code{family = "cox"} this is a
#' \code{\link[survival]{Surv}} object, or equivalently an \eqn{n \times 2} matrix / data frame
#' whose first column is the observed time and whose second column is the event indicator.
#' @param mod.idx A non-empty list of disjoint column-index vectors for all modalities in the
#' concatenated data matrix \code{X}. Every column of \code{X} must be assigned to exactly one modality.
#' @param family A description of the error distribution and link function to be used in the model.
#' With \code{robust = "none"} the Gaussian (\code{"gaussian"}) and Binomial (\code{"binomial"})
#' families with canonical links and the Cox proportional hazards model (\code{"cox"}) are available.
#' With \code{robust = "mom"} the Gaussian (\code{"gaussian"}) and Negative-Binomial
#' (\code{"negative.binomial"}) families are available.
#' @param robust Estimator for the modality information gain. \code{"none"} (default) uses the
#' standard maximum-likelihood / partial-likelihood estimator with closed-form non-central
#' chi-square confidence intervals. \code{"mom"} uses the Median-of-Means gradient-descent
#' (MoM-GD) estimator, which is robust to heavy-tailed responses, with basic bootstrap
#' confidence intervals.
#' @param iter Specifies whether to perform iterative SIS. For the Cox family this toggles between
#' a single BIC-driven screening pass (\code{iter=FALSE}) and the full iterative (ISIS) procedure
#' (\code{iter=TRUE}). The default is \code{iter=TRUE}.
#' @param penalty Specifies the type of penalty to be used in the variable selection and
#' inference procedure.
#' Options include \code{'MCP'}, \code{'SCAD'}, and \code{'lasso'}. The default is \code{penalty='SCAD'}.
#' @param tune Specifies the method for selecting the optimal tuning parameters in (I)SIS and
#' penalized likelihood procedure. Options include \code{'bic'}, \code{'ebic'} and \code{'aic'}.
#' The default is \code{tune='bic'}.
#' @param lambda A user-specified finite, positive, strictly decreasing sequence of at least two
#' lambda values for penalized likelihood
#' procedure. By default, a sequence of values of length \code{nlambda} is automatically computed and
#' equally spaced on the log scale.
#' @param nlambda The number of lambda values, an integer of at least 2. The default is 100.
#' @param conf.level Confidence level, a finite number strictly between 0 and 1. The default is
#' \code{conf.level=0.95}.
#' @param CI.type A string specifying the type of the confidence interval. Options include
#' \code{'two.sided'} and \code{'one.sided'}. The default is \code{CI.type='two.sided'}.
#' @param CI Logical; whether to compute confidence intervals. When \code{robust = "none"} the
#' default is \code{TRUE} (closed-form non-central chi-square intervals). When
#' \code{robust = "mom"} the default is \code{FALSE}; set \code{CI = TRUE} to run the basic
#' bootstrap. If \code{CI = FALSE}, only point estimates are returned (CI bounds are \code{NA}).
#' @param nboot Number of bootstrap resamples (at least 2) used for the MoM basic bootstrap confidence interval
#' (used only when \code{robust = "mom"} and \code{CI = TRUE}). The default is 200.
#' @param nblock Number of Median-of-Means blocks used by the MoM-GD estimator (used only when
#' \code{robust = "mom"}). Must be an integer between 1 and \code{nrow(X)}. The rows of \code{X}
#' are partitioned into \code{nblock} contiguous,
#' near-equal blocks. The default is 5.
#' @param theta Negative-Binomial dispersion parameter (used only when
#' \code{family = "negative.binomial"}), a finite positive number treated as fixed in the MoM fit. If \code{NULL}
#' (default), it is estimated once from the screened design.
#' @param trace Specifies whether to print out logs of iterations in SIS procedure. The default is
#' \code{trace=FALSE}.
#'
#' @return An object with S3 class "mglm.test" containing:
#' \item{sel.idx}{ List of indices of selected features by (I)SIS in each modality.}
#' \item{num.nonzeros}{Number of selected features by (I)SIS in each modality.}
#' \item{ERE}{Point estimation of ERE for each modality.}
#' \item{ERE.CI.L}{Lower bound of the confidence interval of ERE for each modality}
#' \item{ERE.CI.U}{Upper bound of the confidence interval of ERE for each modality}
#' \item{R2}{Point estimate of pseudo-\eqn{R^2} for each modality.}
#' \item{R2.CI.L}{Lower bound of the confidence interval of pseudo-\eqn{R^2} for each modality}
#' \item{R2.CI.U}{Upper bound of the confidence interval of pseudo-\eqn{R^2} for each modality}
#' \item{conf.level}{ Level of confidence intervals.}
#' @export
#'
#' @importFrom MASS mvrnorm glm.nb
#' @importFrom SIS SIS
#' @importFrom glmnet glmnet
#' @importFrom glmnet cv.glmnet
#' @importFrom ncvreg ncvreg
#' @importFrom ncvreg cv.ncvreg
#' @importFrom MBESS conf.limits.nc.chisq
#' @importFrom survival coxph Surv
#' @importFrom dplyr case_when
#' @importFrom utils capture.output

#' @examples
#' ## Example 1: Linear model
#' data(data_linear_model)
#' X <- data_linear_model$X
#' y <- data_linear_model$y
#' mod.idx <- data_linear_model$mod.idx
#' test <- mglm.test(X = X, y = y, mod.idx = mod.idx, family = "gaussian",
#'                iter = TRUE, penalty = "SCAD", tune = "bic",
#'                conf.level = 0.95, CI.type = "one.sided")
#' summary(test)
#'
#'
#' ## Example 2: Logistic regression
#' data(data_logistic_model)
#' X <- data_logistic_model$X
#' y <- data_logistic_model$y
#' mod.idx <- data_logistic_model$mod.idx
#' test <- mglm.test(X = X, y = y, mod.idx = mod.idx, family = "binomial",
#'                iter = TRUE, penalty = "SCAD", tune = "bic",
#'                conf.level = 0.95, CI.type = "two.sided")
#' sum.test <- summary(test)
#'
#'
#' ## Example 3: Cox proportional hazards model
#' \donttest{
#' data(data_cox_model)
#' X <- data_cox_model$X
#' y <- data_cox_model$y          # two-column matrix: (time, status)
#' mod.idx <- data_cox_model$mod.idx
#' test <- mglm.test(X = X, y = y, mod.idx = mod.idx, family = "cox",
#'                iter = TRUE, penalty = "SCAD", tune = "ebic",
#'                conf.level = 0.95, CI.type = "two.sided")
#' summary(test)
#' }
#'
#'
#' ## Example 4: Robust (MoM) estimation under heavy-tailed responses
#' data(data_nb_model)
#' X <- data_nb_model$X
#' y <- data_nb_model$y
#' mod.idx <- data_nb_model$mod.idx
#' ## Point estimates only (CI = FALSE is the default when robust = "mom"):
#' test <- mglm.test(X = X, y = y, mod.idx = mod.idx,
#'                family = "negative.binomial", robust = "mom",
#'                iter = FALSE, penalty = "SCAD", tune = "bic",
#'                nblock = 5, theta = data_nb_model$theta)
#' summary(test)
#'
#'
mglm.test <- function(X, y, mod.idx,
                 family = c("gaussian", "binomial", "cox", "negative.binomial"),
                 robust = c("none", "mom"), iter = TRUE,
                 penalty = c("SCAD", "MCP", "lasso"),
                 tune = c("bic", "ebic", "aic"),
                 lambda = NULL, nlambda = 100,
                 conf.level = 0.95, CI.type = c("two.sided", "one.sided"),
                 CI = NULL, nboot = 200, nblock = 5, theta = NULL,
                 trace = FALSE){

  ###########################################################################################
  # Input argument checks and modifications
  ###########################################################################################

  family = match.arg(family)
  robust = match.arg(robust)
  penalty = match.arg(penalty)
  tune = match.arg(tune)
  CI.type = match.arg(CI.type)

  if (is.null(X) || is.null(y))
    stop("X and y must both be supplied.")
  if (!is.matrix(X) || !is.numeric(X))
    stop("X must be a numeric matrix.")
  if (nrow(X) < 2L || ncol(X) < 1L)
    stop("X must contain at least two rows and one column.")
  if (anyNA(X) || any(!is.finite(X)))
    stop("X must not contain missing or non-finite values.")

  nobs = nrow(X)
  nvars = ncol(X)

  if (!is.list(mod.idx) || length(mod.idx) == 0L)
    stop("mod.idx must be a non-empty list of modality column indices.")
  valid_modality <- vapply(mod.idx, function(idx) {
    is.numeric(idx) && length(idx) > 0L && !anyNA(idx) &&
      all(is.finite(idx)) && all(idx == floor(idx)) &&
      all(idx >= 1L & idx <= nvars) && !anyDuplicated(idx)
  }, logical(1))
  if (!all(valid_modality))
    stop("Each element of mod.idx must contain unique integer indices between 1 and ncol(X).")
  all_modality_indices <- unlist(mod.idx, use.names = FALSE)
  if (anyDuplicated(all_modality_indices))
    stop("Modalities in mod.idx must be disjoint.")
  if (!identical(sort(as.integer(all_modality_indices)), seq_len(nvars)))
    stop("mod.idx must assign every column of X to exactly one modality.")

  is_single_logical <- function(z) is.logical(z) && length(z) == 1L && !is.na(z)
  is_integer_count <- function(z, minimum) {
    is.numeric(z) && length(z) == 1L && !is.na(z) && is.finite(z) &&
      z == floor(z) && z >= minimum
  }

  if (!is_single_logical(iter))
    stop("iter must be a single non-missing logical value.")
  if (!is_single_logical(trace))
    stop("trace must be a single non-missing logical value.")
  if (!is.numeric(conf.level) || length(conf.level) != 1L ||
      is.na(conf.level) || !is.finite(conf.level) ||
      conf.level <= 0 || conf.level >= 1)
    stop("conf.level must be a single finite number strictly between 0 and 1.")
  if (!is_integer_count(nlambda, 2L))
    stop("nlambda must be a single integer greater than or equal to 2.")
  if (!is_integer_count(nboot, 2L))
    stop("nboot must be a single integer greater than or equal to 2.")
  if (!is_integer_count(nblock, 1L) || nblock > nobs)
    stop("nblock must be a single integer between 1 and nrow(X).")
  if (!is.null(lambda)) {
    if (!is.numeric(lambda) || length(lambda) < 2L || anyNA(lambda) ||
        any(!is.finite(lambda)) || any(lambda <= 0) || any(diff(lambda) >= 0))
      stop("lambda must be a finite, positive, strictly decreasing numeric sequence of length at least 2.")
  }
  if (!is.null(theta) &&
      (!is.numeric(theta) || length(theta) != 1L || is.na(theta) ||
       !is.finite(theta) || theta <= 0))
    stop("theta must be NULL or a single finite positive number.")

  ## Default CI behaviour depends on the estimator: closed-form CIs are cheap
  ## for the non-robust path (default on), whereas the robust path needs a
  ## bootstrap (default off unless the user asks for it).
  if (is.null(CI)) CI = (robust == "none")
  if (!is_single_logical(CI))
    stop("CI must be a single logical value (TRUE or FALSE).")

  ## Family must be compatible with the chosen estimator.
  if (robust == "none") {
    if (!(family %in% c("gaussian", "binomial", "cox")))
      stop("With robust='none', 'family' must be 'gaussian', 'binomial', or 'cox'. ",
           "The Negative-Binomial family is only available with robust='mom'.")
  } else { ## robust == "mom"
    if (!(family %in% c("gaussian", "negative.binomial")))
      stop("With robust='mom', 'family' must be 'gaussian' or 'negative.binomial'. ",
           "Median-of-Means estimation is not implemented for family='",
           family, "'.")
  }

  ## For the Cox model the response must be a survival object. Accept either a
  ## 'Surv' object or a two-column (time, status) matrix / data frame.
  if (family == "cox") {
    if (inherits(y, "Surv")) {
      if (nrow(y) != nobs)
        stop("X and y must contain the same number of observations.")
      if (anyNA(y) || any(!is.finite(unclass(y))))
        stop("The Cox response must not contain missing or non-finite values.")
      ysurv = y
    } else {
      ymat = as.matrix(y)
      if (!is.numeric(ymat) || ncol(ymat) != 2)
        stop("For family='cox', 'y' must be a 'Surv' object or a two-column matrix/data.frame with columns (time, status).")
      if (nrow(ymat) != nobs)
        stop("X and y must contain the same number of observations.")
      if (anyNA(ymat) || any(!is.finite(ymat)))
        stop("The Cox response must not contain missing or non-finite values.")
      if (any(ymat[, 1] < 0))
        stop("Cox follow-up times must be non-negative.")
      if (!all(ymat[, 2] %in% c(0, 1)))
        stop("Cox event indicators must contain only 0 and 1.")
      ysurv = Surv(ymat[, 1], ymat[, 2])
    }
    yuse = ysurv
  } else {
    if (is.matrix(y) || is.data.frame(y)) {
      ymat = as.matrix(y)
      if (!is.numeric(ymat) || ncol(ymat) != 1L)
        stop("For non-Cox families, y must be a numeric or logical vector or a one-column matrix/data frame.")
      if (nrow(ymat) != nobs)
        stop("X and y must contain the same number of observations.")
      y = as.numeric(ymat[, 1])
    } else {
      if ((!is.numeric(y) && !is.logical(y)) || !is.null(dim(y)))
        stop("For non-Cox families, y must be a numeric or logical vector or a one-column matrix/data frame.")
      if (length(y) != nobs)
        stop("For non-Cox families, y must have nrow(X) observations.")
      y = as.numeric(y)
    }
    if (anyNA(y) || any(!is.finite(y)))
      stop("y must not contain missing or non-finite values.")
    if (family == "binomial") {
      if (!all(y %in% c(0, 1)) || length(unique(y)) != 2L)
        stop("For family='binomial', y must contain both 0 and 1 and no other values.")
    }
    if (family == "negative.binomial" &&
        (any(y < 0) || any(y != floor(y))))
      stop("For family='negative.binomial', y must contain non-negative integer counts.")
    yuse = y
  }

  nMod = length(mod.idx)
  sel.idx = list()
  ERE = c()
  num.nonzeros = c()
  ERE.CI.L = c()
  ERE.CI.U = c()


  ## Working family used for screening and penalised selection. The Negative-
  ## Binomial mean model shares the log link with the Poisson, so a Poisson
  ## working likelihood is used to screen/select features; MoM-GD then supplies
  ## the robust NB coefficient estimates on the selected set.
  work.family <- if (robust == "mom" && family == "negative.binomial") "poisson" else family

  ## Conduct (I)SIS for dimension reduction
  if(family == "cox"){
    ## (Iterative) sure independence screening for the Cox model
    var_id0 <- screen_cox(X, yuse, iter = iter, trace = trace)
  }else{
    if(trace){
      sis <- SIS(x=X, y=y, family=work.family, iter = iter, tune = tune)
    }else{
      invisible(capture.output(sis <- SIS(x=X, y=y, family=work.family, iter = iter, tune = tune)))
    }
    var_id0 <- sis$ix
  }
  if(length(var_id0)==0){
    warning("No variables selected after SIS")
  }

  ## Negative-Binomial dispersion (fixed for MoM-GD). Estimate once on the
  ## screened design if the user did not supply it.
  if (robust == "mom" && family == "negative.binomial" && is.null(theta)) {
    theta <- if (length(var_id0) != 0)
      est_theta_nb(y, X[, var_id0, drop = FALSE]) else est_theta_nb(y, matrix(0, length(y), 0))
  }

  for(i in seq_len(nMod)){
    Mbeta = setdiff(1:ncol(X),mod.idx[[i]])

    ## partially penalized regularization procedure
    ##
    ## Skipped when screening retained fewer than two features: a single
    ## screened feature is kept as-is for every penalty. The penalized step
    ## could not remove it anyway when it belongs to the conditioning set
    ## (penalty.factor = 0), a one-variable penalized fit is degenerate, and
    ## glmnet (the lasso backend) rejects one-column design matrices outright.
    pen.factor = as.numeric(var_id0 %in% Mbeta)
    if(length(var_id0) >= 2 && sum(pen.factor)!=0){
      ## coerce to a matrix so ncvsurv / coxph handle a single screened feature
      Xsel = as.matrix(X[, var_id0, drop = FALSE])
      tunefit = tune.fit.adj(Xsel, yuse, family=work.family , penalty = penalty, tune=tune,
                             lambda = lambda, nlambda = nlambda,
                             penalty.factor = pen.factor)
      var_id = var_id0[tunefit$ix]   # empty when the penalized fit selects nothing
    }else{
      var_id = var_id0
    }

    ## Estimation of the ERE / H_m
    if(robust == "mom"){
      fit = mom_est(X, y, var_id, Mbeta, family, theta = theta, nblock = nblock,
                    CI = CI, nboot = nboot, conf.level = conf.level, CI.type = CI.type)
    }else{
      fit = mod_est(X, yuse, var_id, Mbeta, family, conf.level = conf.level,
                    CI.type = CI.type, CI = CI)
    }
    sel.idx[[i]] <- var_id[!(var_id %in% Mbeta)]
    num.nonzeros <- c(num.nonzeros,fit$num.nonzeros)
    ERE <- c(ERE,fit$Th)
    ERE.CI.L <- c(ERE.CI.L,fit$CI[1])
    ERE.CI.U <- c(ERE.CI.U,fit$CI[2])
  }

  R2 = (1 - exp(-ERE))
  R2.CI.L = (1-exp(-ERE.CI.L))
  R2.CI.U = (1-exp(-ERE.CI.U))

  result = list(sel.idx=sel.idx, num.nonzeros=num.nonzeros,
                ERE=ERE, ERE.CI.L=ERE.CI.L, ERE.CI.U=ERE.CI.U,
                R2=R2, R2.CI.L=R2.CI.L, R2.CI.U=R2.CI.U,
                conf.level=conf.level)
  class(result) <- "mglm.test"
  return(result)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_simu.R
\docType{data}
\name{data_linear_model}
\alias{data_linear_model}
\title{Example Dataset}
\format{
A data object that contains
\describe{
  \item{\code{y}}{A vector of 200 observations of continuous outcomes.}
  \item{\code{X}}{A 200 \eqn{\times} 600 matrix containing all training data.}
  \item{\code{mod.idx}}{A list of modality indices.}
}
}
\usage{
data_linear_model
}
\description{
A toy dataset to demonstrate running this package on multimodal linear models.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/multiModTest.R
\name{mglm.test}
\alias{mglm.test}
\title{Modality Assessment in Multimodal Generalized Linear Models}
\usage{
mglm.test(
  X,
  y,
  mod.idx,
  family = c("gaussian", "binomial", "cox", "negative.binomial"),
  robust = c("none", "mom"),
  iter = TRUE,
  penalty = c("SCAD", "MCP", "lasso"),
  tune = c("bic", "ebic", "aic"),
  lambda = NULL,
  nlambda = 100,
  conf.level = 0.95,
  CI.type = c("two.sided", "one.sided"),
  CI = NULL,
  nboot = 200,
  nblock = 5,
  theta = NULL,
  trace = FALSE
)
}
\arguments{
\item{X}{The finite numeric \eqn{n \times p} data matrix consisting of features from all modalities.}

\item{y}{The response. For \code{family = "gaussian"} or \code{"binomial"} this is the
length-\eqn{n} vector (or one-column matrix/data frame) of finite responses; binomial responses
must contain both 0 and 1.
Negative-binomial responses must be non-negative integer counts. For \code{family = "cox"} this is a
\code{\link[survival]{Surv}} object, or equivalently an \eqn{n \times 2} matrix / data frame
whose first column is the observed time and whose second column is the event indicator.}

\item{mod.idx}{A non-empty list of disjoint column-index vectors for all modalities in the
concatenated data matrix \code{X}. Every column of \code{X} must be assigned to exactly one modality.}

\item{family}{A description of the error distribution and link function to be used in the model.
With \code{robust = "none"} the Gaussian (\code{"gaussian"}) and Binomial (\code{"binomial"})
families with canonical links and the Cox proportional hazards model (\code{"cox"}) are available.
With \code{robust = "mom"} the Gaussian (\code{"gaussian"}) and Negative-Binomial
(\code{"negative.binomial"}) families are available.}

\item{robust}{Estimator for the modality information gain. \code{"none"} (default) uses the
standard maximum-likelihood / partial-likelihood estimator with closed-form non-central
chi-square confidence intervals. \code{"mom"} uses the Median-of-Means gradient-descent
(MoM-GD) estimator, which is robust to heavy-tailed responses, with basic bootstrap
confidence intervals.}

\item{iter}{Specifies whether to perform iterative SIS. For the Cox family this toggles between
a single BIC-driven screening pass (\code{iter=FALSE}) and the full iterative (ISIS) procedure
(\code{iter=TRUE}). The default is \code{iter=TRUE}.}

\item{penalty}{Specifies the type of penalty to be used in the variable selection and
inference procedure.
Options include \code{'MCP'}, \code{'SCAD'}, and \code{'lasso'}. The default is \code{penalty='SCAD'}.}

\item{tune}{Specifies the method for selecting the optimal tuning parameters in (I)SIS and
penalized likelihood procedure. Options include \code{'bic'}, \code{'ebic'} and \code{'aic'}.
The default is \code{tune='bic'}.}

\item{lambda}{A user-specified finite, positive, strictly decreasing sequence of at least two
lambda values for penalized likelihood
procedure. By default, a sequence of values of length \code{nlambda} is automatically computed and
equally spaced on the log scale.}

\item{nlambda}{The number of lambda values, an integer of at least 2. The default is 100.}

\item{conf.level}{Confidence level, a finite number strictly between 0 and 1. The default is
\code{conf.level=0.95}.}

\item{CI.type}{A string specifying the type of the confidence interval. Options include
\code{'two.sided'} and \code{'one.sided'}. The default is \code{CI.type='two.sided'}.}

\item{CI}{Logical; whether to compute confidence intervals. When \code{robust = "none"} the
default is \code{TRUE} (closed-form non-central chi-square intervals). When
\code{robust = "mom"} the default is \code{FALSE}; set \code{CI = TRUE} to run the basic
bootstrap. If \code{CI = FALSE}, only point estimates are returned (CI bounds are \code{NA}).}

\item{nboot}{Number of bootstrap resamples (at least 2) used for the MoM basic bootstrap confidence interval
(used only when \code{robust = "mom"} and \code{CI = TRUE}). The default is 200.}

\item{nblock}{Number of Median-of-Means blocks used by the MoM-GD estimator (used only when
\code{robust = "mom"}). Must be an integer between 1 and \code{nrow(X)}. The rows of \code{X}
are partitioned into \code{nblock} contiguous,
near-equal blocks. The default is 5.}

\item{theta}{Negative-Binomial dispersion parameter (used only when
\code{family = "negative.binomial"}), a finite positive number treated as fixed in the MoM fit. If \code{NULL}
(default), it is estimated once from the screened design.}

\item{trace}{Specifies whether to print out logs of iterations in SIS procedure. The default is
\code{trace=FALSE}.}
}
\value{
An object with S3 class "mglm.test" containing:
\item{sel.idx}{ List of indices of selected features by (I)SIS in each modality.}
\item{num.nonzeros}{Number of selected features by (I)SIS in each modality.}
\item{ERE}{Point estimation of ERE for each modality.}
\item{ERE.CI.L}{Lower bound of the confidence interval of ERE for each modality}
\item{ERE.CI.U}{Upper bound of the confidence interval of ERE for each modality}
\item{R2}{Point estimate of pseudo-\eqn{R^2} for each modality.}
\item{R2.CI.L}{Lower bound of the confidence interval of pseudo-\eqn{R^2} for each modality}
\item{R2.CI.U}{Upper bound of the confidence interval of pseudo-\eqn{R^2} for each modality}
\item{conf.level}{ Level of confidence intervals.}
}
\description{
Provides statistical inference for modality-specific information gain in
multimodal GLMs. Estimates ERE and pseudo-\eqn{R^2} with confidence intervals using
Sure Independence Screening for variable selection and penalized likelihood for inference.
}
\examples{
## Example 1: Linear model
data(data_linear_model)
X <- data_linear_model$X
y <- data_linear_model$y
mod.idx <- data_linear_model$mod.idx
test <- mglm.test(X = X, y = y, mod.idx = mod.idx, family = "gaussian",
               iter = TRUE, penalty = "SCAD", tune = "bic",
               conf.level = 0.95, CI.type = "one.sided")
summary(test)


## Example 2: Logistic regression
data(data_logistic_model)
X <- data_logistic_model$X
y <- data_logistic_model$y
mod.idx <- data_logistic_model$mod.idx
test <- mglm.test(X = X, y = y, mod.idx = mod.idx, family = "binomial",
               iter = TRUE, penalty = "SCAD", tune = "bic",
               conf.level = 0.95, CI.type = "two.sided")
sum.test <- summary(test)


## Example 3: Cox proportional hazards model
\donttest{
data(data_cox_model)
X <- data_cox_model$X
y <- data_cox_model$y          # two-column matrix: (time, status)
mod.idx <- data_cox_model$mod.idx
test <- mglm.test(X = X, y = y, mod.idx = mod.idx, family = "cox",
               iter = TRUE, penalty = "SCAD", tune = "ebic",
               conf.level = 0.95, CI.type = "two.sided")
summary(test)
}


## Example 4: Robust (MoM) estimation under heavy-tailed responses
data(data_nb_model)
X <- data_nb_model$X
y <- data_nb_model$y
mod.idx <- data_nb_model$mod.idx
## Point estimates only (CI = FALSE is the default when robust = "mom"):
test <- mglm.test(X = X, y = y, mod.idx = mod.idx,
               family = "negative.binomial", robust = "mom",
               iter = FALSE, penalty = "SCAD", tune = "bic",
               nblock = 5, theta = data_nb_model$theta)
summary(test)


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_simu.R
\docType{data}
\name{data_cox_model}
\alias{data_cox_model}
\title{Example Dataset}
\format{
A data object that contains
\describe{
  \item{\code{y}}{A 200 \eqn{\times} 2 matrix of survival outcomes, with columns
  \code{time} (the observed follow-up time) and \code{status} (the event
  indicator: 1 = event, 0 = censored). It may be passed directly to
  \code{\link{mglm.test}}, or converted with \code{\link[survival]{Surv}}.}
  \item{\code{X}}{A 200 \eqn{\times} 800 matrix containing all training data.}
  \item{\code{mod.idx}}{A list of modality indices: columns 1--400 form
  modality 1 and columns 401--800 form modality 2.}
}
}
\usage{
data_cox_model
}
\description{
A toy dataset to demonstrate running this package on multimodal Cox
proportional hazards models. Survival times follow a Cox model with
independent right censoring (about 27\% of the observations are censored).
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_simu.R
\docType{data}
\name{data_nb_model}
\alias{data_nb_model}
\title{Example Dataset}
\format{
A data object that contains
\describe{
  \item{\code{y}}{A vector of 300 observations of non-negative counts.}
  \item{\code{X}}{A 300 \eqn{\times} 600 matrix containing all training data.}
  \item{\code{mod.idx}}{A list of modality indices: columns 1--200, 201--400,
  and 401--600 form modalities 1, 2, and 3.}
  \item{\code{theta}}{The true Negative-Binomial dispersion used to generate
  the counts, \eqn{\theta = 0.5}. It may be supplied to \code{\link{mglm.test}}
  through the \code{theta} argument; otherwise it is estimated from the data.}
}
}
\usage{
data_nb_model
}
\description{
A toy dataset to demonstrate robust (Median-of-Means) estimation on
multimodal Negative-Binomial models with heavy-tailed responses. The counts
are strongly overdispersed (variance-to-mean ratio near 19, with roughly
half of the observations equal to zero).
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.mglm.test.R
\name{summary.mglm.test}
\alias{summary.mglm.test}
\alias{print.summary.mglm.test}
\title{Summary method for objects of class "mglm.test"}
\usage{
\method{summary}{mglm.test}(object, ...)

\method{print}{summary.mglm.test}(x, ...)
}
\arguments{
\item{object}{An \code{mglm.test} object.}

\item{...}{Additional arguments that could be passed to \code{summary.mglm.test}.}

\item{x}{A \code{summary.mglm.test} object.}
}
\value{
An object with S3 class \code{summary.mglm.test}. The class has its own print
method and contains the following list of elements.
\item{sum.ERE}{The summary table of point estimate and confidence interval
of ERE for each modality.}
\item{sum.R2}{The summary table of point estimate and confidence interval
of pseudo-\eqn{R^2} for each modality.}
\item{conf.level}{ Level of confidence intervals.}
\item{sel.mod}{ Index of the most informative modality.}
}
\description{
Summary method for objects of class "mglm.test"
}
